@@ -6,7 +6,7 @@ RDF::TrineShortcuts - totally unauthorised module for cheats and charlatans
 
 =head1 VERSION
 
-0.06
+0.07
 
 =head1 SYNOPSIS
 
@@ -35,11 +35,18 @@ use RDF::Trine '0.116';
 use RDF::Trine::Serializer;
 use RDF::Query '2.200';
 use RDF::Query::Client;
+use URI;
 use URI::file;
 
-our @ISA     = qw(Exporter);
-our @EXPORT  = qw(rdf_parse rdf_string rdf_query);
-our $VERSION = '0.06';
+our @ISA      = qw(Exporter);
+our %EXPORT_TAGS = (
+	default => [qw(rdf_parse rdf_string rdf_query)],
+	nodes   => [qw(rdf_node rdf_literal rdf_blank rdf_resource)],
+	all     => [qw(rdf_parse rdf_string rdf_query rdf_node rdf_literal rdf_blank rdf_resource rdf_statement)],
+	);
+our @EXPORT    = ( @{ $EXPORT_TAGS{'default'} } );
+our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
+our $VERSION   = '0.07';
 my $Has;
 
 BEGIN
@@ -49,13 +56,13 @@ BEGIN
 	{
 		$@ = undef;
 		eval "use $package;";
-		$Has->{$package} unless $@;
+		$Has->{$package} = 1 unless $@;
 	}
 }
 
 =head1 DESCRIPTION
 
-This module exports three functions which simplify frequently performed 
+This module exports four functions which simplify frequently performed 
 tasks using L<RDF::Trine> and L<RDF::Query>.
 
 In addition, because it calls "use RDF::Trine", "use RDF::Query", and
@@ -117,13 +124,26 @@ sub rdf_parse
 		$context = RDF::Trine::Node::Resource->new("$context");
 	}
 	
+	if ($input =~ /^([A-Za-z0-9_]+)(\;[^\r\n]*)?$/
+        && UNIVERSAL::isa("RDF::Trine::Store::$1", "RDF::Trine::Store"))
+	{
+		die "need RDF::Trine 0.118 (for new_with_string support)"
+			unless RDF::Trine::Store->can('new_with_string');
+		
+		$input = RDF::Trine::Store->new_with_string($input);
+	}
+
+	if (UNIVERSAL::isa($input, 'RDF::Trine::Store'))
+	{
+		$input = RDF::Trine::Model->new($input);
+	}
+
 	if (UNIVERSAL::isa($input, 'RDF::Trine::Model'))
 	{
 		$input = $input->as_stream;
 	}
 
-	if (UNIVERSAL::isa($input, 'RDF::Trine::Iterator')
-	and $input->is_graph)
+	if (UNIVERSAL::isa($input, 'RDF::Trine::Iterator'))
 	{
 		while (my $st = $input->next)
 		{
@@ -179,9 +199,9 @@ sub rdf_parse
 		$input->close;
 		$input = $_;
 	}
-
+	
 	if ($Has->{'RDF::RDFa::Parser'}
-	and ($type =~ m'atom'i or ((!defined $type) and $input =~ m'http://www.w3.org/2005/Atom')))
+	and (($type||'') =~ m'atom'i or ((!defined $type) and $input =~ m'http://www.w3.org/2005/Atom')))
 	{
 		my $opts = RDF::RDFa::Parser::OPTS_ATOM();
 		$opts->{'atom_parser'} = 1;
@@ -190,14 +210,14 @@ sub rdf_parse
 		return $model;
 	}
 	elsif ($Has->{'XML::Atom::OWL'}
-	and ($type =~ m'atom'i or ((!defined $type) and $input =~ m'http://www.w3.org/2005/Atom')))
+	and (($type||'') =~ m'atom'i or ((!defined $type) and $input =~ m'http://www.w3.org/2005/Atom')))
 	{
 		my $p = XML::Atom::OWL->new($input, $base, undef, $model->_store);
 		$p->consume;
 		return $model;
 	}
 	elsif ($Has->{'RDF::RDFa::Parser'}
-	and $type =~ m'svg'i)
+	and ($type||'') =~ m'svg'i)
 	{
 		my $opts = RDF::RDFa::Parser::OPTS_SVG();
 		my $p = RDF::RDFa::Parser->new($input, $base, $opts, $model->_store);
@@ -205,7 +225,7 @@ sub rdf_parse
 		return $model;
 	}
 	elsif ($Has->{'XRD::Parser'}
-	and ($type =~ m'xrd'i or ((!defined $type) and $input =~ m'http://docs.oasis-open.org/ns/xri/xrd-1.0')))
+	and (($type||'') =~ m'xrd'i or ((!defined $type) and $input =~ m'http://docs.oasis-open.org/ns/xri/xrd-1.0')))
 	{
 		my $p = XRD::Parser->new($input, $base, undef, $model->_store);
 		$p->consume;
@@ -214,38 +234,45 @@ sub rdf_parse
 
 	my $parser;
 
-	if ($type =~ /rdf.?xml/i)
+	if (defined $type)
 	{
-		$parser = RDF::Trine::Parser->new('RDFXML');
+		if ($type =~ /rdf.?xml/i)
+		{
+			$parser = RDF::Trine::Parser->new('RDFXML');
+		}
+		elsif ($type =~ /json/i)
+		{
+			$parser = RDF::Trine::Parser->new('RDFJSON');
+		}
+		elsif ($type =~ /turtle/i)
+		{
+			$parser = RDF::Trine::Parser->new('Turtle');
+		}
+		elsif ($type =~ /ntriple/i)
+		{
+			$parser = RDF::Trine::Parser->new('NTriples');
+		}
+		elsif ($type =~ /nquad/i)
+		{
+			$parser = RDF::Trine::Parser->new('NQuads');
+		}
+		elsif ($type =~ /(xhtml|rdfa)/i)
+		{
+			$parser = RDF::Trine::Parser->new('RDFa');
+		}
+		else
+		{
+			eval { $parser = RDF::Trine::Parser->new($type); }
+		}
 	}
-	elsif ($type =~ /json/i)
-	{
-		$parser = RDF::Trine::Parser->new('RDFJSON');
-	}
-	elsif ($type =~ /turtle/i)
-	{
-		$parser = RDF::Trine::Parser->new('Turtle');
-	}
-	elsif ($type =~ /ntriple/i)
-	{
-		$parser = RDF::Trine::Parser->new('NTriples');
-	}
-	elsif ($type =~ /nquad/i)
-	{
-		$parser = RDF::Trine::Parser->new('NQuads');
-	}
-	elsif ($type =~ /(xhtml|rdfa)/i)
-	{
-		$parser = RDF::Trine::Parser->new('RDFa');
-	}
-	elsif ($type)
-	{
-		eval { $parser = RDF::Trine::Parser->new($type); }
-	}
-
+	
 	unless ($parser)
 	{
-		if ($input =~ /<rdf:RDF/)
+		if ($input =~ /^\s*\{/s)
+		{
+			$parser = RDF::Trine::Parser->new('RDFJSON');
+		}
+		elsif ($input =~ /<rdf:RDF/)
 		{
 			$parser = RDF::Trine::Parser->new('RDFXML');
 		}
@@ -261,10 +288,6 @@ sub rdf_parse
 		{
 			$parser = RDF::Trine::Parser->new('Turtle');
 		}
-		elsif ($input =~ /^\s*\{/s)
-		{
-			$parser = RDF::Trine::Parser->new('RDFJSON');
-		}
 		else
 		{
 			$parser = RDF::Trine::Parser->new('RDFXML');
@@ -294,13 +317,35 @@ $format is the format to use. One of 'RDFXML' (the default), 'RDFJSON',
 'Turtle', 'Canonical NTriples' or 'NTriples'. If $format is not one of the above, 
 then the function will try to guess what you meant.
 
+Preferred namespace names can be provided as a named argument:
+
+ print rdf_string($model, 'turtle',
+    namespaces => { foo=>'http://example.com/vocabs/foo#' });
+
 =cut
 
 sub rdf_string
 {
 	my $model = shift;
 	my $fmt   = shift || 'RDFXML';
+	my %args  = @_;
 	my $s;
+	
+	$args{'namespaces'} = {
+		dc   => 'http://purl.org/dc/terms/',
+		dc11 => 'http://purl.org/dc/elements/1.1/',
+		doap => 'http://usefulinc.com/ns/doap#',
+		foaf => 'http://xmlns.com/foaf/0.1/',
+		geo  => 'http://www.w3.org/2003/01/geo/wgs84_pos#',
+		owl  => 'http://www.w3.org/2002/07/owl#',
+		rdf  => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#',
+		rdfs => 'http://www.w3.org/2000/01/rdf-schema#',
+		sioc => 'http://rdfs.org/sioc/ns#',
+		skos => 'http://www.w3.org/2004/02/skos/core#',
+		v    =>	'http://www.w3.org/2006/vcard/ns#',
+		xsd  => 'http://www.w3.org/2001/XMLSchema#',
+		}
+		unless defined $args{'namespaces'};
 
 	unless (UNIVERSAL::isa($model, 'RDF::Trine::Model'))
 	{
@@ -321,7 +366,7 @@ sub rdf_string
 	}
 	elsif ($fmt =~ /turtle/i || $fmt =~ /n(otation)\s*3/i)
 	{
-		$s = RDF::Trine::Serializer::Turtle->new;
+		$s = RDF::Trine::Serializer::Turtle->new(namespaces=>$args{'namespaces'});
 	}
 	elsif ($fmt =~ /n\-?t/i)
 	{
@@ -329,7 +374,7 @@ sub rdf_string
 	}
 	else
 	{
-		$s = RDF::Trine::Serializer::RDFXML->new;
+		$s = RDF::Trine::Serializer::RDFXML->new(namespaces=>$args{'namespaces'});
 	}
 
 	return $s->serialize_model_to_string($model);
@@ -407,6 +452,209 @@ sub rdf_query
 	}
 }
 
+=item C<< rdf_node($value, %args) >>
+
+Creates an RDF::Trine::Node object.
+
+Will attempt to automatically determine whether $value is a blank node,
+resource or literal, but an optional named argument 'type' can be used
+to explicitly indicate this.
+
+For literals, named arguments 'datatype' and 'lang' are allowed. If
+'datatype' is not a URI, then it's assumed to be an XSD datatype.
+
+ $node = rdf_node("Hello", type=>'literal', lang=>'en');
+
+For resources, the named argument 'base' is allowed.
+
+This function is not exported by default, but can be exported using
+the tag ':nodes' or ':all'.
+
+ use RDF::TrineShortcuts qw(:default :nodes);
+
+=cut
+
+sub rdf_node
+{
+	my $value = shift;
+	my %args  = @_;
+	
+	return $value
+		if UNIVERSAL::isa($value, 'RDF::Trine::Node');
+	
+	if (defined ($args{'datatype'} || $args{'lang'} || $args{'language'}))
+	{
+		$args{'type'} ||= 'literal';
+	}
+	elsif ($value =~ /^_:(.*)$/)
+	{
+		$args{'type'} ||= 'blank';
+		$value = $1 if $args{'type'} =~ /(blank|bnode)/i;
+	}
+	elsif ($value =~ /^[a-z0-9\+\-]+:\S*$/i)
+	{
+		$args{'type'} ||= 'resource';
+	}
+	elsif (UNIVERSAL::isa($value, 'DateTime'))
+	{
+		$value              = "$value";
+		$args{'type'}     ||= 'literal';
+		$args{'datatype'} ||= 'dateTime' unless $args{'lang'};
+	}
+	elsif (UNIVERSAL::isa($value, 'URI'))
+	{
+		$value              = "$value";
+		$args{'type'}     ||= 'resource';
+		$args{'datatype'} ||= 'anyURI' unless $args{'lang'};
+	}
+	else
+	{
+		$args{'type'} ||= 'literal';
+	}
+	
+	if ($args{'type'} =~ /(uri|url|resource)/i)
+	{
+		if ($args{'base'})
+		{
+			$value = URI->new_abs("$value", $args{'base'});
+			$value = "$value";
+		}
+		
+		return RDF::Trine::Node::Resource->new($value);
+	}
+	
+	elsif ($args{'type'} =~ /(bnode|blank)/i)
+	{
+		return RDF::Trine::Node::Blank->new($value);
+	}
+	
+	else
+	{
+		if (defined $args{'datatype'}
+		&& length $args{'datatype'}
+		&& $args{'datatype'} !~ /^[a-z0-9\+\-]+:\S*$/i)
+		{
+			$args{'datatype'} = sprintf('http://www.w3.org/2001/XMLSchema#%s',
+				$args{'datatype'});
+		}
+		
+		return RDF::Trine::Node::Literal->new($value, $args{'lang'}||$args{'language'}, $args{'datatype'});
+	}
+}
+
+=item C<< rdf_literal($value, %args) >>,
+C<< rdf_blank($value, %args) >>, 
+C<< rdf_resource($value, %args) >>
+
+Shortcuts for rdf_node($value, type=>'literal', %args) and so on.
+
+These functions are not exported by default, but can be exported using
+the tag ':nodes' or ':all'.
+
+ use RDF::Trine qw(:all);
+
+=cut
+
+sub rdf_literal
+{
+	my $value = shift;
+	return rdf_node($value, type=>'literal', @_);
+}
+
+sub rdf_blank
+{
+	my $value = shift;
+	return rdf_node($value, type=>'blank', @_);
+}
+
+sub rdf_resource
+{
+	my $value = shift;
+	return rdf_node($value, type=>'resource', @_);
+}
+
+=item C<< rdf_statement($s, $p, $o, [$g]) >>, 
+C<< rdf_statement($ntriple, [$g]) >>
+
+Returns an RDF::Trine::Statement. Parameters $s, $p, $o and $g
+can each be either a plain string that could be passed to rdf_node,
+or an arrayref of rdf_node parameters, or an RDF::Trine::Node.
+
+$ntriple is a single N-Triples statement.
+
+This function is not exported by default, but can be exported using
+the tag ':all'.
+
+ use RDF::Trine qw(:all);
+
+=cut
+
+sub rdf_statement
+{
+	my ($s, $p, $o, $g) = @_;
+	
+	if (!defined ($o||$g))
+	{
+		(my $ntriple, $g, $p) = ($s, $p, undef);
+		$ntriple .= ' .' unless $ntriple =~ /\.[\s\r\n]*$/;
+		my $model = rdf_parse($ntriple, type=>'ntriples');
+		my $iter  = $model->as_stream;
+		if (my $st = $iter->next)
+		{
+			return $st;
+		}
+		else
+		{
+			die "Could not parse N-Triples statement";
+		}
+	}
+
+	if (ref $s eq 'ARRAY')
+	{
+		$s = rdf_node(@$s);
+	}
+	else
+	{
+		$s = rdf_node($s);
+	}
+
+	if (ref $p eq 'ARRAY')
+	{
+		$p = rdf_resource(@$p);
+	}
+	else
+	{
+		$p = rdf_resource($p);
+	}
+
+	if (ref $o eq 'ARRAY')
+	{
+		$o = rdf_node(@$o);
+	}
+	else
+	{
+		$o = rdf_node($o);
+	}
+
+	if (ref $g eq 'ARRAY')
+	{
+		$g = rdf_node(@$g);
+	}
+	elsif (defined $g)
+	{
+		$g = rdf_node($g);
+	}
+
+	if (defined $g)
+	{
+		return RDF::Trine::Statement->new($s, $p, $o, $g);
+	}
+	else
+	{
+		return RDF::Trine::Statement->new($s, $p, $o);
+	}
+}
+
 1;
 
 __END__
@@ -423,8 +671,10 @@ L<RDF::Trine>, L<RDF::Query>, L<RDF::Query::Client>.
 
 L<http://www.perlrdf.org/>.
 
-This module is distributed with two command-line RDF tools. L<trapper> is
-an RDF fetcher/parser/serialiser; L<toquet> is a SPARQL query tool.
+This module is distributed with three command-line RDF tools.
+L<trapper> is an RDF fetcher/parser/serialiser;
+L<toquet> is a SPARQL query tool;
+L<trist> is an RDF statistics tool.
 
 =head1 AUTHOR
 
